@@ -58,10 +58,28 @@ end
 
 function mixin_generic_plot_attributes()
     return @DocumentedAttributes begin
+        """
+        Controls the inheritance or directly sets the transformations of a plot.
+        Transformations include the transform function and model matrix as generated
+        by `translate!(...)`, `scale!(...)` and `rotate!(...)`. They can be set
+        directly by passing a `Transformation()` object or inherited from the
+        parent plot or scene. Inheritance options include:
+        - `:automatic`: Inherit transformations if the parent and child `space` is compatible
+        - `:inherit`: Inherit transformations
+        - `:inherit_model`: Inherit only model transformations
+        - `:inherit_transform_func`: Inherit only the transform function
+        - `:nothing`: Inherit neither, fully disconnecting the child's transformations from the parent
+
+        Another option is to pass arguments to the `transform!()` function which
+        then get applied to the plot. For example `transformation = (:xz, 1.0)`
+        which rotates the `xy` plane to the `xz` plane and translates by `1.0`.
+        For this inheritance defaults to `:automatic` but can also be set through
+        e.g. `(:nothing, (:xz, 1.0))`.
+        """
         transformation = :automatic
         "Sets a model matrix for the plot. This overrides adjustments made with `translate!`, `rotate!` and `scale!`."
         model = automatic
-        "Controls whether the plot will be rendered or not."
+        "Controls whether the plot gets rendered or not."
         visible = true
         "Adjusts how the plot deals with transparency. In GLMakie `transparency = true` results in using Order Independent Transparency."
         transparency = false
@@ -75,7 +93,12 @@ function mixin_generic_plot_attributes()
         depth_shift = 0.0f0
         "Sets the transformation space for box encompassing the plot. See `Makie.spaces()` for possible inputs."
         space = :data
-        "Adjusts whether the plot is rendered with fxaa (anti-aliasing, GLMakie only)."
+        """
+        Adjusts whether the plot is rendered with fxaa (fast approximate anti-aliasing, GLMakie only).
+        Note that some plots implement a better native anti-aliasing solution (scatter, text, lines).
+        For them `fxaa = true` generally lowers quality. Plots that show smoothly interpolated data
+        (e.g. image, surface) may also degrade in quality as `fxaa = true` can cause blurring.
+        """
         fxaa = true
         "Sets a callback function `(plot, index, position) -> string` which replaces the default label generated by DataInspector."
         inspector_label = automatic
@@ -128,8 +151,8 @@ function colormap_attributes(attr)
     )
 end
 
-function mixin_colormap_attributes()
-    return @DocumentedAttributes begin
+function mixin_colormap_attributes(; kwargs...)
+    attr = @DocumentedAttributes begin
         """
         Sets the colormap that is sampled for numeric `color`s.
         `PlotUtils.cgrad(...)`, `Makie.Reverse(any_colormap)` can be used as well, or any symbol from ColorBrewer or PlotUtils.
@@ -151,6 +174,7 @@ function mixin_colormap_attributes()
         "The alpha value of the colormap or color attribute. Multiple alphas like in `plot(alpha=0.2, color=(:red, 0.5)`, will get multiplied."
         alpha = 1.0
     end
+    return filter_attributes!(attr; kwargs...)
 end
 
 """
@@ -340,6 +364,11 @@ Plots a surface, where `(x, y)` define a grid whose heights are the entries in `
 @recipe Surface (x::VecOrMat{<:FloatType}, y::VecOrMat{<:FloatType}, z::VecOrMat{<:FloatType}) begin
     "Can be set to an `Matrix{<: Union{Number, Colorant}}` to color surface independent of the `z` component. If `color=nothing`, it defaults to `color=z`. Can also be a `Makie.AbstractPattern`."
     color = nothing
+    """
+    Applies a "material capture" texture to the generated mesh. A matcap encodes
+    lighting and color data of a material on a circular texture which is sampled
+    based on normal vectors.
+    """
     matcap = nothing
     "Inverts the normals generated for the surface. This can be useful to illuminate the other side of the surface."
     invert_normals = false
@@ -393,7 +422,14 @@ Creates a connected line plot for each element in `(x, y, z)`, `(x, y)` or `posi
     joinstyle = @inherit joinstyle
     "Sets the minimum inner join angle below which miter joins truncate. See also `Makie.miter_distance_to_angle`."
     miter_limit = @inherit miter_limit
-    "Sets which attributes to cycle when creating multiple plots."
+    """
+    Sets which attributes to cycle when creating multiple plots. The values to
+    cycle through are defined by the parent Theme. Multiple cycled attributes can
+    be set by passing a vector. Elements can
+    - directly refer to a cycled attribute, e.g. `:color`
+    - map a cycled attribute to a palette attribute, e.g. `:linecolor => :color`
+    - map multiple cycled attributes to a palette attribute, e.g. `[:linecolor, :markercolor] => :color`
+    """
     cycle = [:color]
     mixin_generic_plot_attributes()...
     mixin_colormap_attributes()...
@@ -423,7 +459,14 @@ Plots a line for each pair of points in `(x, y, z)`, `(x, y)`, or `positions`.
     linestyle = nothing
     "Sets the type of linecap used, i.e. :butt (flat with no extrusion), :square (flat with 1 linewidth extrusion) or :round."
     linecap = @inherit linecap
-    "Sets which attributes to cycle when creating multiple plots."
+    """
+    Sets which attributes to cycle when creating multiple plots. The values to
+    cycle through are defined by the parent Theme. Multiple cycled attributes can
+    be set by passing a vector. Elements can
+    - directly refer to a cycled attribute, e.g. `:color`
+    - map a cycled attribute to a palette attribute, e.g. `:linecolor => :color`
+    - map multiple cycled attributes to a palette attribute, e.g. `[:linecolor, :markercolor] => :color`
+    """
     cycle = [:color]
     mixin_generic_plot_attributes()...
     mixin_colormap_attributes()...
@@ -449,7 +492,20 @@ Plots a 3D or 2D mesh. Supported `mesh_object`s include `Mesh` types from [Geome
     color = @inherit patchcolor
     "sets whether colors should be interpolated"
     interpolate = true
+    """
+    Sets which attributes to cycle when creating multiple plots. The values to
+    cycle through are defined by the parent Theme. Multiple cycled attributes can
+    be set by passing a vector. Elements can
+    - directly refer to a cycled attribute, e.g. `:color`
+    - map a cycled attribute to a palette attribute, e.g. `:linecolor => :color`
+    - map multiple cycled attributes to a palette attribute, e.g. `[:linecolor, :markercolor] => :color`
+    """
     cycle = [:color => :patchcolor]
+    """
+    Applies a "material capture" texture to the generated mesh. A matcap encodes
+    lighting and color data of a material on a circular texture which is sampled
+    based on normal vectors.
+    """
     matcap = nothing
     """
     Sets a transform for uv coordinates, which controls how a texture is mapped to a mesh.
@@ -508,10 +564,16 @@ Plots a marker for each element in `(x, y, z)`, `(x, y)`, or `positions`.
     Sets the font to be used for character markers
     """
     font = "default"
-    uv_offset_width = (0.0, 0.0, 0.0, 0.0)
     "Sets the space in which `markersize` is given. See `Makie.spaces()` for possible inputs"
     markerspace = :pixel
-    "Sets which attributes to cycle when creating multiple plots"
+    """
+    Sets which attributes to cycle when creating multiple plots. The values to
+    cycle through are defined by the parent Theme. Multiple cycled attributes can
+    be set by passing a vector. Elements can
+    - directly refer to a cycled attribute, e.g. `:color`
+    - map a cycled attribute to a palette attribute, e.g. `:linecolor => :color`
+    - map multiple cycled attributes to a palette attribute, e.g. `[:linecolor, :markercolor] => :color`
+    """
     cycle = [:color]
     "Enables depth-sorting of markers which can improve border artifacts. Currently supported in GLMakie only."
     depthsorting = false
@@ -544,6 +606,14 @@ Plots a mesh for each element in `(x, y, z)`, `(x, y)`, or `positions` (similar
     markersize = 0.1
     "Sets the rotation of the mesh. A numeric rotation is around the z-axis, a `Vec3f` causes the mesh to rotate such that the the z-axis is now that vector, and a quaternion describes a general rotation. This can be given as a Vector to apply to each scattered mesh individually."
     rotation = 0.0
+    """
+    Sets which attributes to cycle when creating multiple plots. The values to
+    cycle through are defined by the parent Theme. Multiple cycled attributes can
+    be set by passing a vector. Elements can
+    - directly refer to a cycled attribute, e.g. `:color`
+    - map a cycled attribute to a palette attribute, e.g. `:linecolor => :color`
+    - map multiple cycled attributes to a palette attribute, e.g. `[:linecolor, :markercolor] => :color`
+    """
     cycle = [:color]
     """
     Sets a transform for uv coordinates, which controls how a texture is mapped to the scattered mesh.
@@ -723,12 +793,33 @@ Plots polygons, which are defined by
     For custom patterns have a look at [`Makie.Linestyle`](@ref).
     """
     linestyle = nothing
+    """
+    Sets the type of line cap used for outlines. Options are `:butt` (flat without
+    extrusion), `:square` (flat with half a linewidth extrusion) or `:round`.
+    """
     linecap = @inherit linecap
+    """
+    Controls the rendering of outline corners. Options are `:miter` for sharp corners,
+    `:bevel` for "cut off" corners, and `:round` for rounded corners. If the corner angle
+    is below `miter_limit`, `:miter` is equivalent to `:bevel` to avoid long spikes.
+    """
     joinstyle = @inherit joinstyle
+    """
+    Sets the minimum inner join angle below which miter line joins truncate.
+    See also `Makie.miter_distance_to_angle`.
+    """
     miter_limit = @inherit miter_limit
-
+    "Controls whether lights affect the polygon."
     shading = false
 
+    """
+    Sets which attributes to cycle when creating multiple plots. The values to
+    cycle through are defined by the parent Theme. Multiple cycled attributes can
+    be set by passing a vector. Elements can
+    - directly refer to a cycled attribute, e.g. `:color`
+    - map a cycled attribute to a palette attribute, e.g. `:linecolor => :color`
+    - map multiple cycled attributes to a palette attribute, e.g. `[:linecolor, :markercolor] => :color`
+    """
     cycle = [:color => :patchcolor]
     """
     Depth shift of stroke plot. This is useful to avoid z-fighting between the stroke and the fill.
