@@ -8,6 +8,10 @@ struct RichText
     end
 end
 
+Base.:(==)(a::RichText, b::RichText) = a.type == b.type && a.children == b.children && a.attributes == b.attributes
+
+Base.hash(a::RichText, b::UInt) = hash(a.type, hash(a.children, hash(a.attributes, b)))
+
 function check_textsize_deprecation(@nospecialize(dictlike))
     return if haskey(dictlike, :textsize)
         throw(ArgumentError("`textsize` has been renamed to `fontsize` in Makie v0.19. Please change all occurrences of `textsize` to `fontsize` or revert back to an earlier version."))
@@ -372,13 +376,6 @@ function register_text_computations!(attr::ComputeGraph)
     # And :glyphcollection if applicable
     compute_glyph_collections!(attr)
 
-    map!(attr, [:text_blocks, :positions], :text_positions) do blocks, pos
-        if length(blocks) != length(pos)
-            error("Text blocks and positions have different lengths: $(length(blocks)) != $(length(pos)). Please use `update!(plot_object; arg1/arg2/text/position/color/etc...) to update multiple attributes together.")
-        end
-        return [p for (b, p) in zip(blocks, pos) for i in b]
-    end
-
     map!(attr, [:atlas, :glyphindices, :font_per_char], :sdf_uv) do atlas, gi, fonts
         return glyph_uv_width!.((atlas,), gi, fonts)
     end
@@ -408,7 +405,15 @@ function register_text_computations!(attr::ComputeGraph)
     end
     # TODO: remapping positions to be per glyph first generates quite a few
     # redundant transform applications and projections in CairoMakie
-    register_position_transforms!(attr, input_name = :text_positions, transformed_name = :positions_transformed)
+    register_position_transforms!(attr, input_name = :positions, transformed_name = :positions_transformed)
+
+    map!(attr, [:text_blocks, :positions_transformed_f32c], :per_char_positions_transformed_f32c) do blocks, pos
+        if length(blocks) != length(pos)
+            error("Text blocks and positions have different lengths: $(length(blocks)) != $(length(pos)). Please use `update!(plot_object; arg1/arg2/text/position/color/etc...) to update multiple attributes together.")
+        end
+        return [p for (b, p) in zip(blocks, pos) for i in b]
+    end
+
     return
 end
 
@@ -445,7 +450,7 @@ function tex_linesegments!(plot)
         markerspace_positions = _project(preprojection * model_f32c, positions, clip_planes, space)
         # TODO: avoid repeated apply_transform and use block_idx?
         return map(linesegments, indices) do seg, (block_idx, glyph_idx)
-            return seg + markerspace_positions[glyph_idx]
+            return seg + markerspace_positions[block_idx]
         end
     end
 
@@ -479,6 +484,36 @@ function register_markerspace_positions!(plot::Text, ::Type{OT} = Point3f; kwarg
     )
 end
 
+struct PerCharIterator{T}
+    blocks::Vector{UnitRange{Int64}}
+    data::Vector{T}
+    is_per_block::Bool
+end
+function PerCharIterator(blocks, data)
+    return PerCharIterator(blocks, data, length(blocks) == length(data))
+end
+
+function Base.iterate(iter::PerCharIterator, state = (1, 1))
+    char_idx, block_idx = state
+    if block_idx > length(iter.blocks) || char_idx > last(last(iter.blocks))
+        return nothing
+    end
+
+    if iter.is_per_block
+
+        if char_idx in iter.blocks[block_idx]
+            return iter.data[block_idx], (char_idx + 1, block_idx)
+        else
+            return iterate(iter, (char_idx, block_idx + 1))
+        end
+    else
+        return iter.data[char_idx], (char_idx + 1, 0)
+    end
+end
+
+Base.length(iter::PerCharIterator) = last(last(iter.blocks))
+
+
 # TODO: anything per-string should include lines?
 
 function register_raw_glyph_boundingboxes!(plot)
@@ -535,11 +570,11 @@ function register_glyph_boundingboxes!(plot)
         register_markerspace_positions!(plot)
         map!(
             plot.attributes,
-            [:raw_glyph_boundingboxes, :marker_offset, :text_rotation, :markerspace_positions],
+            [:raw_glyph_boundingboxes, :marker_offset, :text_rotation, :text_blocks, :markerspace_positions],
             :glyph_boundingboxes
-        ) do bbs, origins, rotations, positions
+        ) do bbs, origins, rotations, blocks, positions
 
-            return map(bbs, origins, rotations, positions) do bb, o, rotation, position
+            return map(bbs, origins, rotations, PerCharIterator(blocks, positions)) do bb, o, rotation, position
                 glyphbb3 = Rect3d(to_ndim(Point3d, origin(bb), 0), to_ndim(Point3d, widths(bb), 0))
                 return rotate_bbox(glyphbb3, rotation) + o + position
             end
@@ -561,6 +596,47 @@ which need to be transformed to `markerspace`.
 glyph_boundingboxes(plot) = register_glyph_boundingboxes!(plot)[]::Vector{Rect3d}
 glyph_boundingboxes_obs(plot) = ComputePipeline.get_observable!(register_glyph_boundingboxes!(plot))
 
+# target: rotation aware layouting, e.g. Axis ticks, Menu, ...
+function register_raw_string_boundingboxes!(plot)
+    if !haskey(plot.attributes, :raw_string_boundingboxes)
+        register_raw_glyph_boundingboxes!(plot)
+        # To consider newlines (and word_wrap_width) we need to include origins.
+        # To not include rotation we need to strip it from origins
+        map!(
+            plot.attributes, [:text_blocks, :raw_glyph_boundingboxes, :glyph_origins, :text_rotation, :linesegments, :linewidths, :lineindices],
+            :raw_string_boundingboxes
+        ) do blocks, bbs, origins, rotation, segments, linewidths, lineindices
+
+            text_bbs = map(blocks) do idxs
+                output = Rect3d()
+                for i in idxs
+                    glyphbb = bbs[i]
+                    glyphbb3 = Rect3d(to_ndim(Point3d, origin(glyphbb), 0), to_ndim(Point3d, widths(glyphbb), 0))
+                    ms_bb = rotate_bbox(glyphbb3, rotation[i]) + origins[i]
+                    output = update_boundingbox(output, ms_bb)
+                end
+                return output
+            end
+
+            for (pos, lw, (block_idx, glyph_idx)) in zip(segments, linewidths, lineindices)
+                bb = Rect3d(to_ndim(Point3d, pos, 0) .- 0.5lw, Vec3d(lw))
+                text_bbs[block_idx] = update_boundingbox(text_bbs[block_idx], bb)
+            end
+
+            return text_bbs
+        end
+    end
+    return plot.raw_string_boundingboxes
+end
+
+"""
+    raw_string_boundingboxes(plot::Text)
+
+Returns the markerspace string boundingboxes without including `positions` and `offset`.
+Rotation is included. Lines from LaTeXStrings are included.
+"""
+raw_string_boundingboxes(plot) = register_raw_string_boundingboxes!(plot)[]::Vector{Rect3d}
+raw_string_boundingboxes_obs(plot) = ComputePipeline.get_observable!(register_raw_string_boundingboxes!(plot))
 
 # target: rotation aware layouting, e.g. Axis ticks, Menu, ...
 function register_fast_string_boundingboxes!(plot)
@@ -574,7 +650,7 @@ function register_fast_string_boundingboxes!(plot)
         ) do blocks, bbs, origins, rotation, segments, linewidths, lineindices
 
             text_bbs = map(blocks) do idxs
-                output = Rect3d()
+                output = Rect3d(Point3d(NaN), Vec3d(0))
                 for i in idxs
                     glyphbb = bbs[i]
                     glyphbb3 = Rect3d(to_ndim(Point3d, origin(glyphbb), 0), to_ndim(Point3d, widths(glyphbb), 0))
@@ -613,15 +689,16 @@ function register_string_boundingboxes!(plot)
         # project positions to markerspace, add them
         map!(
             plot.attributes,
-            [:text_blocks, :fast_string_boundingboxes, :markerspace_positions],
+            [:fast_string_boundingboxes, :markerspace_positions],
             :string_boundingboxes
-        ) do text_blocks, bbs, positions
+        ) do bbs, positions
 
-            return map(enumerate(text_blocks)) do (i, idxs)
-                if isempty(idxs)
-                    return Rect3d(Point3d(NaN), Vec3d(0))
-                else
-                    return bbs[i] + positions[first(idxs)]
+            return map(bbs, positions) do bb, pos
+                mini = minimum(bb)
+                if isfinite(mini)
+                    return bb + pos
+                else # empty bboxes end up as Rect3d(Point3d(Inf), Vec3d(-Inf))
+                    return Rect3d(pos, Vec3d(0))
                 end
             end
         end
@@ -839,6 +916,10 @@ where both scripts are right-aligned against the following text.
 """
 left_subsup(args...; kwargs...) = RichText(:leftsubsup, args...; kwargs...)
 
+Base.:*(x::RichText, y::AbstractString) = rich(x, y)
+Base.:*(x::AbstractString, y::RichText) = rich(x, y)
+Base.:*(x::RichText, y::RichText) = rich(x, y)
+
 export rich, subscript, superscript, subsup, left_subsup
 
 struct GlyphState
