@@ -55,13 +55,13 @@ macro Block(_name::Union{Expr, Symbol}, body::Expr = Expr(:block))
 
     push!(fields_vector, constructor)
 
-    docs_placeholder = gensym()
+    docs_placeholder = Symbol("#__", name, "_docs_placeholder")
 
     q = quote
         # This part is as far as I know the only way to modify the docstring on top of the
         # recipe, so that we can offer the convenience of automatic augmented docstrings
         # but combine them with the simplicity of using a normal docstring.
-        # The trick is to mark some variable (in this case a gensymmed placeholder) with the
+        # The trick is to mark some variable with the
         # Core.@__doc__ macro, which causes this variable to get assigned the docstring on top
         # of the @recipe invocation. From there, it can then be retrieved, modified, and later
         # attached to plotting function by using @doc again. We also delete the binding to the
@@ -496,6 +496,23 @@ function Base.show(io::IO, ::T) where {T <: Block}
     return print(io, "$T()")
 end
 
+function Base.show(io::IO, ::MIME"text/plain", ax::AbstractAxis)
+    nplots = length(ax.scene.plots)
+    kind = typeof(ax)
+    println(io, "$kind with $nplots plots:")
+
+    for (i, p) in enumerate(ax.scene.plots)
+        println(io, (i == nplots ? " ┗━ " : " ┣━ ") * string(typeof(p)))
+    end
+    return
+end
+
+function Base.show(io::IO, ax::AbstractAxis)
+    nplots = length(ax.scene.plots)
+    kind = typeof(ax)
+    return print(io, "$kind ($nplots plots)")
+end
+
 # fallback if block doesn't need specific clean up
 free(::Block) = nothing
 
